#!/bin/bash

# make sure we are in the directory containing this script
SCRIPTDIR=`dirname $0`
cd $SCRIPTDIR

case "$1" in
    remap_page_range)
        shift
        #
        # Determine the number of arguments expected by remap_page_range.
        #
        echo "#include <linux/mm.h>
        int do_test_remap_page_range(void) {
           pgprot_t pgprot;
           remap_page_range(NULL, 0L, 0L, 0L, pgprot);
        }" > conftest.c

        gcc -c conftest.c -o conftest.5.o $* -D__KERNEL__ > /dev/null 2>&1

        echo "#include <linux/mm.h>
        int do_test_remap_page_range(void) {
           pgprot_t pgprot;
           remap_page_range(0L, 0L, 0L, pgprot);
        }" > conftest.c

        gcc -c conftest.c -o conftest.4.o $* -D__KERNEL__ > /dev/null 2>&1

        if   test -f conftest.4.o; then
          echo "4"
          rm -f conftest.{c,4.o}
        elif test -f conftest.5.o; then
          echo "5"
          rm -f conftest.{c,5.o}
        else
          #
          # We couldn't determine the number of arguments expected by the
          # remap_page_range function.
          #
          rm -f conftest.{c,*.o}
          exit 1
        fi
    ;;

    cc_sanity_check)
        shift
        #
        # Verify that the same compiler is used for the kernel and kernel
        # module.
        #
        if test -n "$IGNORE_CC_MISMATCH" -o -n "$SYSDIR" -o -n "$SYSINCLUDE"; then
          #
          # The user chose to disable the gcc sanity test (which may or
          # may not be wise) or is building the module for a kernel not
          # currently running, which renders our test meaningless.
          #
          exit 0
        fi
        
        # usage: conftest.sh [cc_sanity_check] [full_output|just_msg] [$(CC)]

        OUTPUT=$1
        shift

        rm -f gcc-version-check
        $* gcc-version-check.c -o gcc-version-check > /dev/null 2>&1
        if [ -f gcc-version-check ]; then
            PROC_VERSION=`cat /proc/version`
            MSG=`./gcc-version-check "$PROC_VERSION"`
            RET=$?
            rm -f gcc-version-check
        else
            MSG="Could not compile gcc-version-check.c"
            RET=1
        fi

        if [ "$RET" != "0" ]; then
            #
            # The gcc version check failed
            #
            
            if [ "$OUTPUT" = "full_output" ]; then
                echo "";
                echo "gcc-version-check failed:";
                echo "";
                echo "$MSG" | fmt -w 60
                echo "";
                echo "If you know what you are doing and want to override";
                echo "the gcc version check, you can do so by setting the";
                echo "IGNORE_CC_MISMATCH environment variable to \"1\".";
                echo "";
                echo "In any other case, set the CC environment variable";
                echo "to the name of the compiler that was used to compile";
                echo "the kernel.";
                echo ""
                echo -en "\033[1;31m";
                echo -e  "*** Failed cc sanity check. Bailing out! ***";
                echo -en "\033[0m";
                echo "";
            else
                echo "$MSG"
            fi
            exit 1;
        else
            exit 0
        fi
    ;;

    kernel_patch_level)
        shift
        #
        # Determine the kernel's major patch level; this is only done if we
        # aren't told by KBUILD.
        #
        echo $(cat "$1"/Makefile | grep "PATCHLEVEL =" | cut -d " " -f 3)
        exit 0
    ;;

    suser_sanity_check)
        shift
        #
        # Determine the caller's user id to determine if we have sufficient
        # privileges for the requested operation.
        #
        if test $(id -ur) != 0; then
            echo "";
            echo "Please run \"make install\" as root.";
            echo "";
            echo -en "\033[1;31m";
            echo -e  "*** Failed super-user sanity check. Bailing out! ***";
            echo -en "\033[0m";
            exit 1
        else
            exit 0
        fi
    ;;

    rmmod_sanity_check)
        shift
        #
        # Make sure that any currently loaded NVIDIA kernel module can be
        # unloaded.
        #
        if PATH="${PATH}:/bin/sbin" lsmod | grep -wq "$1"; then
          PATH="${PATH}:/bin/sbin" rmmod "$1" >& /dev/null
        fi

        if PATH="${PATH}:/bin/sbin" lsmod | grep -wq "$1"; then
            #
            # The NVIDIA kernel module is still loaded, most likely because
            # it is busy.
            #
            echo "";
            echo "Unable to remove existing NVIDIA kernel module.";
            echo "Please be sure you have exited X before attempting";
            echo "to install the NVIDIA kernel module.";
            echo "";
            echo -en "\033[1;31m";
            echo -e  "*** Failed rmmod sanity check. Bailing out! ***";
            echo -en "\033[0m";
            exit 1
        else
            exit 0
        fi
    ;;

    select_makefile)
        shift
        #
        # Select which Makefile to use based on the version of the
        # kernel we are building against: use the kbuild Makefile for
        # 2.6 and newer kernels, and the old Makefile for kernels older
        # than 2.6.
        #
        if [ "$SYSSRC" ]; then
            KERNEL_INCLUDE=$SYSSRC/include
        elif [ "$SYSINCLUDE" ]; then
            KERNEL_INCLUDE=$SYSINCLUDE
        else
            KERNEL_INCLUDE="/lib/modules/`uname -r`/build/include"
        fi
        
        echo "#include \"linux/version.h\"
        int main() {
          if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)) {
            return 0;
          } else {
            return 1;
          }
        }" > conftest.c
        
        gcc conftest.c -o conftest -nostdinc \
            -I $KERNEL_INCLUDE > /dev/null 2>&1
        
        if [ -f conftest ]; then
            rm -f Makefile
            ./conftest > /dev/null 2>&1
            if [ $? = "0" ]; then
                ln -s Makefile.kbuild Makefile
            else
                ln -s Makefile.nvidia Makefile
            fi
            rm -f conftest conftest.c
            exit 0
        else
            echo "Unable to determine kernel version."
            exit 1
        fi
    ;;

    get_uname)
        #
        # print UTS_RELEASE from the kernel sources that we are building
        # against; if an error occurs, give up and just return `uname -r`
        #
        shift
        
        if [ "$SYSSRC" ]; then
            KERNEL_INCLUDE=$SYSSRC/include
        elif [ "$SYSINCLUDE" ]; then
            KERNEL_INCLUDE=$SYSINCLUDE
        else
            KERNEL_INCLUDE="/lib/modules/`uname -r`/build/include"
        fi
        
        echo "#include \"linux/version.h\"
        int main() { printf(\"%s\n\", UTS_RELEASE); return 0; }" \
            > conftest.c
        
        gcc conftest.c -o conftest -nostdinc \
            -I $KERNEL_INCLUDE > /dev/null 2>&1
        
        if [ -f conftest ]; then
            ./conftest
            rm -f conftest conftest.c
            exit 0
        else
            uname -r
            exit 1
        fi
    ;;

esac
